import main
from main import batchRequests
import openai



def simple_openai_call(prompt):
  print(f"in simple openai call with question:  {prompt}")
  model = "gpt-4"
  messages = [
    {
      "role": "system",
      "content": "You are a helpful assistant."
    },
    {
      "role": "user",
      "content": prompt
    },
  ]
  result = openai.ChatCompletion.create(model=model, messages=messages)
  print(f"Result: from open ai for {prompt}, result {result}")
  return result


batch_requests = batchRequests(process_func=simple_openai_call,
                              max_token_capacity=40000,
                              max_request_capacity=200,
                              set_timeout=12000)

list_questions = [
  "What is the difference between a list and a tuple in Python?",
  "How do you iterate over a dictionary in Python?",
  "What is the purpose of the 'self' parameter in a class method?",
  "What are lambda functions in Python?",
  "How do you remove duplicates from a list in Python?",
  "What is the difference between append() and extend() methods in Python lists?",
  "How do you read a file in Python?", "How do you write to a file in Python?",
  "What is the difference between a shallow copy and a deep copy in Python?",
  "How do you convert a string to lowercase in Python?"
]

results = batch_requests.execute(list_questions)

print("\n\n\n")
print(f"Results from reliableGPT {results}")



from setuptools import setup, find_packages

setup(
    name='reliableGPT',
    version='0.2.911',
    description='Error handling and auto-retry library for GPT',
    author='Ishaan Jaffer',
    packages=find_packages(),
    install_requires=[
        'openai',
        'termcolor',
        'klotty',
        'numpy',
        'posthog',
        'tiktoken'
    ],
)



import queue
from concurrent.futures import ThreadPoolExecutor
import threading
from custom_queue import CustomQueue

class APICallHandler():

  def __init__(self, max_token_capacity, max_request_capacity, verbose):
    super().__init__()
    self.verbose = verbose
    self.upperbound = 1000000
    self.q = CustomQueue(max_token_capacity=max_token_capacity,
                         max_request_capacity=max_request_capacity,
                         verbose=verbose)
    self.results = {
    }  #key-value pair, task_id is the key, and the response from openai is the value
    # Define a sentinel value for stopping the worker
    # Start the worker thread
    worker_thread = threading.Thread(target=self.worker)
    worker_thread.start()

  def print_verbose(self, *args):
    if self.verbose:
      print(*args)
    
  def set_upperbound(self, upperbound):
    self.upperbound = upperbound

  def result_handler(self, result, task_id):
    self.results[task_id] = result

  def process_item(self, function, argument, task_id):
    print(f"Processing taskID: {task_id}, for function {function} with input: {argument} ")
    result = function(argument)
    self.result_handler(result, task_id)
    return result

  def run_async(self, items):
    self.print_verbose(items)
    if isinstance(items, list):
      threads = []
      for item in items:
        self.print_verbose(item)
        function, argument, task_id = item
        thread = threading.Thread(target=self.process_item,
                                  args=(function, argument, task_id))
        thread.start()
        threads.append(thread)

      # Wait for all threads to finish
      for thread in threads:
        thread.join()

  # Define worker function that processes items from the queue
  def worker(self):
    count_items = 0
    while True:
      if count_items == self.upperbound:
        break
      items = self.q.get()
      self.print_verbose("gets item from queue: ", items)
      # if items is self.STOP_WORKER:
      #   break  # Exit the loop when the sentinel value is encountered
      self.print_verbose(type(items))
      self.print_verbose(f"items: {items}")
      if isinstance(items, list):
        count_items += len(items)
        self.run_async(items=items)
      else:
        function, argument, task_id = items
        count_items += 1
        result = function(argument)  # Call the function with its argument
        self.print_verbose("result from function arg")
        self.result_handler(
          result, task_id)  # Call the callback function with the result
      self.q.task_done()
      self.print_verbose(f"count_items: {count_items} \n\n self.upperbound: {self.upperbound} ")

  def add_sentinel(self, sentinel_value): 
    self.q.put(sentinel_value)

  def add_task(self, process_func, input, task_id):
    self.q.put((process_func, input, task_id))

  def get_result(self, task_id):
    if task_id in self.results:
      return self.results[task_id]
    else:
      return None

  def get_results(self):
    return self.results


import openai
from main import reliableGPT
import uuid

openai.api_key = ""
from tenacity import retry, stop_after_attempt, wait_random_exponential

# make openAI reliable and safe
openai.ChatCompletion.create = reliableGPT(openai.ChatCompletion.create, user_email= "ishaan@berri.ai", user_token = "AxQgeB3aEDK2B3x4fNG3ZYJRvFfOfQuCTOR83Y_9y5g", send_notification=True)
openai.Completion.create = reliableGPT(openai.Completion.create, user_email= "ishaan@berri.ai", user_token = "AxQgeB3aEDK2B3x4fNG3ZYJRvFfOfQuCTOR83Y_9y5g", send_notification=True)
openai.Embedding.create = reliableGPT(openai.Embedding.create, user_email= "ishaan@berri.ai", user_token = "AxQgeB3aEDK2B3x4fNG3ZYJRvFfOfQuCTOR83Y_9y5g", send_notification=True)

def open_ai_completion_call():
    model = "gpt-4"
    messages = [
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Who won the world series in 2020?"},
        {"role": "assistant", "content": "The Los Angeles Dodgers won the World Series in 2020."},
        {"role": "user", "content": "Where was it played?"}
    ]
    temperature = 0.7
    metadata = {
        "request_uuid": str(uuid.uuid4()),  # Generate a new UUID for each request
        "user": "ishaan@berri.ai"
    }
    response = openai.ChatCompletion.create(model=model, messages=messages, temperature=temperature)
    print(response)


@retry(wait=wait_random_exponential(min=1, max=20), stop=stop_after_attempt(6))
def get_embedding(text, model="text-embedding-ada-002", metadata={}):
  text = text.replace("\n", " ")
  return openai.Embedding.create(input=[text],
                                 model=model, metadata=metadata)["data"][0]["embedding"]

def open_ai_embed_call():
    # openai.Embedding.create = reliableGPT(openai.Embedding.create, user_email= "ishaan@berri.ai", user_token = 'QA5T6lYfzB-8u3gFlC0hxtBZ-TbkJRF_FwrCB8GKTLM', send_notification=True)
    text = """
    Write in less than 25 words a hyper contextualized message about your company and how what your company does could help the visiting company's product offering. Start the message with a lowercase letter.
Reference previous successful chat conversations for the tone and voice. Mix up the language. Try saying something new to each visitor. Make sure it makes sense. Be as specific as possible. If you don't know the person or the company then write a funny
joke based on the person's location. If you don't have the person's location then write a message based on the information of the page the person is on. If you know the person's company mention the person's company in the message. If you don't know the company
then don't mention it. Take into account previous chat history and context around the company and the visitor. Do not include any urls in the response.
At the very end of the message say "(not a bot btw, real person)". Speak succinctly as you would to your best friend.
Your response to the visitor on the site should be as helpful as possible. If you know the company the person works for or the person's name, include the first name or company name in the message.
If the person's name or company is not known, then start the message with "hey!" Craft the style of your message as the best sales development representative in the world would write.
Do not say the same message twice to the person. Start the response with a lowercase letter. Do not start a new conversation or message the person if you've already messaged them within 72 hours.
    """
    metadata = {
                "request_uuid": "40ae3a49-8a01-4c47-9985-6f22bca8c5b6", 
                "user": "ishaan@berri.ai"
    }
    embedding_result = get_embedding(text, metadata=metadata)
    print(embedding_result)
    return embedding_result


import concurrent.futures

def test_multiple_calls():
    model = "gpt-4"
    messages = [
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Who won the world series in 2020?" * 600},
        {"role": "assistant", "content": "The Los Angeles Dodgers won the World Series in 2020."},
        {"role": "user", "content": "Where was it played?"}
    ]
    temperature = 0.7

    error_count = 0
    failure_count = 0  # Track the number of failures

    def call_reliable_openai():
        nonlocal error_count, failure_count
        try:
            print("Making OpenAI Call")

            response = openai.ChatCompletion.create(model=model, messages=messages, temperature=temperature)
            print(response)
            if response and "error" in response:
                error_count += 1
            if response == "Sorry, the OpenAI (GPT) failed":
                failure_count += 1

        except Exception as e:
            print("Exception occurred:", e)
            error_count += 1

    # Create a ThreadPoolExecutor with a maximum of 10 threads
    with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
        # Submit the callable to the executor for each call
        future_calls = [executor.submit(call_reliable_openai) for _ in range(20)]

        # Wait for all the futures to complete
        concurrent.futures.wait(future_calls)

    print(f"Error Count: {error_count}")
    print(f"Fallback response count: {failure_count}")

    if error_count == 0:
        print("All calls executed successfully.")
    else:
        print("Some calls returned errors.")




def test_single_call_bad_key():
    openai.api_key = "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"
    model = "gpt-4"
    messages = [
        {"role": "system", "content": "You are a helpful assistant."},
        {"role": "user", "content": "Who won the Chess championship 2022"*100},
    ]
    temperature = 0.7

    error_count = 0
    failure_count = 0  # Track the number of failures

    try:
        print("Making OpenAI Call")
        response = openai.ChatCompletion.create(model=model, messages=messages, temperature=temperature)
        if response and "error" in response:
            error_count += 1
        if response == "Sorry, the OpenAI (GPT) failed":
            failure_count += 1
    except Exception as e:
        print("Exception occurred:", e)
        error_count += 1

    print(f"Error Count: {error_count}")
    print(f"Fallback response count: {failure_count}")

    if error_count == 0:
        print("All calls executed successfully.")
    else:
        print("Some calls returned errors.")

# test_single_call_bad_key()

def krrish_test_key():
    openai.api_key = "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" 
    completion = openai.ChatCompletion.create(model="gpt-3.5-turbo", messages=[{"role": "user", "content": "Hello world"}])
    print(completion)
# krrish_test_key()
# def test_add_keys():
#     result = main.add_keys(account_email="ishaan1@berri.ai", keys=["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", "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"])
#     print(result)

# def test_delete_keys():
#     result = main.delete_keys(account_email="ishaan@berri.ai", account_token="OwfY1OFqy8fxR0zsJQPW_tcMPo8N7_vP8x3YW-dU9R8")
#     print(result)


# test_add_keys()
# test_delete_keys()


def test_embedding_bad_key():
    from main import reliableGPT
    openai.Embedding.create = reliableGPT(openai.Embedding.create, user_email= "ishaan@berri.ai", user_token = 'QA5T6lYfzB-8u3gFlC0hxtBZ-TbkJRF_FwrCB8GKTLM', send_notification=True)

    openai.api_key = "bad-key"
    def get_embedding(text, model="text-embedding-ada-002"):
        text = text.replace("\n", " ")
        print("text")
        return openai.Embedding.create(input=[text],
                                        model=model)["data"][0]["embedding"]
    result = get_embedding("GM")
    #print(f"This is the embedding response {result}")

#open_ai_completion_call()
#test_embedding_bad_key()
#open_ai_embed_call()
test_multiple_calls()